{{

    Demo for fast Hartley transform.

    This demo requires TV and keyboard.

    The demo simulates sum of 3 sine waves of different frequencies
    and amplitudes, sampled at 8192 Hz. Waveform and transform result
    are displayed on TV. Use Ins key to switch between frequency
    components, Home and End to change frequency of selected component,
    PgUp and PgDn to change amplitude.

}}
CON

    X_TILES = 16
    Y_TILES = 10
    
    bitmap_size = X_TILES  * Y_TILES * 64

	_clkmode = xtal1 + pll16x
	_xinfreq = 5_000_000
    _stack = (bitmap_size * 2 ) >> 2  + 128  'accomodate display memory and stack


	SIN_90		= $0800
	SIN_180		= $1000
	SIN_TABLE	= $E000 >> 1


    bitmap_base = $8000 - bitmap_size
    display_base = bitmap_base - bitmap_size


VAR
    '' 128 samples
    long samples[128]

    word  screen[x_tiles * y_tiles]
    byte f

OBJ
	ht      : "fht"
    kb      : "Keyboard"
    tv      : "TV"
    gr      : "Graphics"

PUB main : i | dx, dy, a, mask, x, y

    '' start up TV and Graphics
    tv_screen := @screen
    tv_colors := @colors
    tv.start(@tvparams)
    gr.start
    gr.setup(X_TILES, Y_TILES, 0, 110, bitmap_base)

    'init tile screen
    repeat dx from 0 to x_tiles - 1
        repeat dy from 0 to y_tiles - 1
            screen[dy * x_tiles + dx] := display_base >> 6 + dy + dx * y_tiles

    '' start FHT cog
    ht.start
    '' start keyboard driver
    kb.start(26, 27)

    '' dra
    updateDisplay
    repeat
        if kb.gotkey
            case kb.getkey
                202:
                    f := (f + 1) // 3
                196:
                    if word[@freq1][f] < 4096
                        word[@freq1][f] += 16
                197:
                    if word[@freq1][f] > 64
                        word[@freq1][f] -= 16
                198:
                    if byte[@a1][f] < 16
                        byte[@a1][f] ++
                199:
                    if byte[@a1][f] > 0
                        byte[@a1][f] --
                other: next
            updateDisplay

PRI updateDisplay : i | s, j
    gr.clear '' clear screen
    gr.colorwidth(1, 0)
    repeat i from 0 to 127 '' fill in the samples with sum of 3 sine waves
        s := 0
        repeat j from 0 to 2
            s += sin($800 + $2000 * i * word[@freq1][j] / 8192)  / 3 * byte[@a1][j] / 16
        samples[i] := s
        ifnot i
            gr.plot(0, s/1536)
        gr.line(i * 2, s / 1536) '' plot the waveform
    '' run fast Hartley transform (1 << 7 = 128 -> transform size)
    ht.transform(@samples, 7, ht#DECIMATE | ht#BUTTERLFIES | ht#POWER | ht#MAGNITUDE, TRUE)
    gr.colorwidth(3, 0)
    gr.colorwidth(3, 1)
    gr.plot(0, -70) '' start transform plot
    repeat i from 1 to 63
        gr.line(i * 4, samples[i] / 512- 70) '' plot transform results, scaling to fit the screen
    '' display frequencies and amplitudes
    repeat i from 0 to 2
        s := i * 70
        j := i*5
        gr.colorwidth((||i==f)*2+1, 0)
        itoa(word[@freq1][i], @str1 + j)
        itoa(byte[@a1][i], @str1 + (j + 5) // 15)
        gr.text(s, -90, @F_str + i * 4)
        gr.text(s + 20, -90, @str1 + j)
        gr.text(s, -106, @A_str + i * 4)
        gr.text(s + 20, -106, @str1 + (j + 5) // 15)
    gr.copy(display_base)


PRI itoa(x, buf) : a
    byte[buf] := 0
    repeat
        bytemove(buf+1, buf, a+1)
        a++
        byte[buf] := "0" + x // 10
        x /= 10
    until x == 0

PRI sin(angle) : tmp
'' Sine
	tmp := angle
	if tmp & SIN_90
		angle := -angle
	angle |= SIN_TABLE
	angle <<= 1
	angle := word[angle]
	if tmp & SIN_180
		angle := -angle
	return angle

DAT
tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode
tv_screen               long    0               'screen
tv_colors               long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog

colors                  long    $5BCA0702

str1                    byte    0[5]
str2                    byte    0[5]
str3                    byte    0[5]
F_str                   byte    "F1=", 0, "F2=", 0, "F3=", 0
A_str                   byte    "A1=", 0, "A2=", 0, "A3=", 0

freq1 word 64
freq2 word 512
freq3 word 2048

a1 byte 12
a2 byte 16
a3 byte 8

