CON

	_clkmode = xtal1 + pll16x
	_xinfreq = 5_000_000

    MIN_BITS = 4
    MAX_BITS = 11

	SIN_90		= $0800
	SIN_180		= $1000
	SIN_TABLE	= $E000 >> 1

VAR

    long samples[2048]
    long working[2048]
    long ma[MAX_BITS - MIN_BITS + 1]
    
    
    
OBJ
    ser : "FullDuplexSerial"
    ht : "fht"
    
PUB main : i | j, bin
    ser.start(31, 30, 0, 115200)
    ht.start
    waitcnt(clkfreq + CNT)
    repeat i from 1 to MAX_BITS + 4
        header(1)
        ser.str(string("Sine (fNyquist/"))
        ser.dec(1 << (i-1))
        ser.str(string(")", $0A))
        header(2)
        repeat j from MIN_BITS to MAX_BITS
            makeSine(1 << i, j)
            measure(j)
    header(string("DC"))
    repeat i from MIN_BITS to MAX_BITS
        makeDC(i)
        measure(i)
    repeat j from 1 to 20
        header(string("Noise"))
        seed := seed_init
        seed_init++
        repeat i from MIN_BITS to MAX_BITS
            makeNoise(i)
            measure(i)
    ser.tx($0A)
    header(string("Worst case"))
    repeat i from MIN_BITS to MAX_BITS
        print(1 << i)
        ser.tx(" ")
        print(ma[i - MIN_BITS])
        ser.tx($0A)
        
PRI makeDC(bits) : i
    repeat i from 0 to 2047
        samples[i] := 1 << (24 - bits)

PRI makeSine(nfDiv, bits) : i | phase
    repeat i from 0 to 1 << bits - 1
        phase := 4096 / nfDiv * i + $800
        if bits > 8
            samples[i] := sin(phase) ~> (bits - 8)
        else
            samples[i] := sin(phase) << (8 - bits )

PRI makeNoise(bits) : i
    repeat i from 0 to 2047
        samples[i] := (?seed)
        if samples[i] < 0
            samples[i] := - (||samples[i] & (1 << (24 - bits)-1))

PRI measure(i) 
    longmove(@working, @samples, 2048)
    print(1 << i)
    ser.tx(" ")
    ht.transform(@working, i, ht#DECIMATE | ht#BUTTERLFIES | ht#POWER | ht#MAGNITUDE, TRUE)
    ma[i - MIN_BITS] #>= ht.getCycles / 80
    print(ht.getCycles / 80)
    ser.tx($0A)

PRI header(str)
    if (str <> 2)
        ser.str(string("====================", $0A))
    if (str > 2)
        ser.str(str)
        ser.tx($0A)
    if str <> 1
        ser.str(string("-------------------", $0A))
        ser.str(string("    Size   Time(uS)", $0A))
        ser.str(string("-------------------", $0A))

PRI print(x) : i
    i := 10000000
    if x => 0
        ser.tx(" ")
    repeat while i
        if ||x < i
            ser.tx(" ")
        i /= 10
    ser.dec(x)

PRI sin(angle) : tmp
'' Sine
	tmp := angle
	if tmp & SIN_90
		angle := -angle
	angle |= SIN_TABLE
	angle <<= 1
	angle := word[angle]
	if tmp & SIN_180
		angle := -angle
	return angle

DAT
seed_init  long 45478
seed        long  0